'use strict';
const { selectCurrentRelation } = require('../relations');
const { capitalizeFirstLetter } = require('../utils/support');
const { setDefaultValue, setCurrentColumnType } = require('./support');
const { additionalSpaces } = require('../utils/constants');
const { itemsForSimpleArray } = require('./constants');

const IMPORT_LINE = `import {
  Entity,
  Column,
  ManyToOne,
  ManyToMany,
  OneToMany,
  OneToOne,
  JoinColumn,
  PrimaryGeneratedColumn,
} from 'typeorm';`;

const IMPORT_POLYMORPHIC_LINE = `import { PolymorphicChildren, PolymorphicParent } from 'typeorm-polymorphic';`;
const IMPORT_POLYMORPHIC_INTERFACE_LINE = `import { PolymorphicChildInterface } from 'typeorm-polymorphic/dist/polymorphic.interface'`;

const entityLine = (name) => `@Entity('${name.toLowerCase()}')`;
const CLOSE_ENTITY = `}`;

const generateImportModel = (modelName) => {
  if (!modelName) {
    return;
  }
  const capitalizeName = capitalizeFirstLetter(modelName);
  return `import { ${capitalizeName} } from '../../${modelName}/entities/${modelName}.entity';`;
};

const generateClassNameEntity = (nameFromModel, polymorphicArr) => {
  if (polymorphicArr.length && polymorphicArr.some(item => typeof item.polymorphic !== 'string')) {
    return `export class ${capitalizeFirstLetter(nameFromModel)} implements PolymorphicChildInterface {`;
  }
  return `export class ${capitalizeFirstLetter(nameFromModel)} {`;
};

const generatePrimaryColumn = (type = 'number') => {
  const uuid = type === 'string';
  return `
  @PrimaryGeneratedColumn(${uuid ? "'uuid'" : ''})
  id: ${type};
  `;
};

const generateColumnSettings = (value, currentColumnType, defaultValue, columnName, optionalName) => {
  const name = optionalName ? optionalName.toLowerCase() : columnName.toLowerCase();

  if (defaultValue === null) {
    return `{ name:'${name}', default: ${value}, nullable: true }`;
  }

  if (typeof defaultValue === 'object') {
    const isArray = value === '[]';
    return `{ name:'${name}', default: ${isArray ? value : JSON.stringify(value)}, type: 'simple-${isArray ? 'array' : 'json'}' }`;
  }

  if (defaultValue !== undefined) {
    return `{ name:'${name}', default: ${value} }`;
  }

  if (currentColumnType === '{}') {
    return `{ name:'${name}', type: 'simple-json' }`;
  }

  if (itemsForSimpleArray.some(item => item === currentColumnType)) {
    return `{ name:'${name}', type: 'simple-array' }`;
  }

  return `{ name:'${name}' }`;
};

const generateDefaultValue = (defaultValue, value, currentColumnType, columnName, optionalName) => {
  return `@Column(${ generateColumnSettings(value, currentColumnType, defaultValue, columnName, optionalName) })`;
};

const generateColumnLines = (
  columnName,
  columnType,
  defaultValue = undefined,
  postgresql = undefined,
  optionalName = undefined,
  ) => {
  const value = setDefaultValue(defaultValue, columnType);
  const currentColumnType = setCurrentColumnType(columnType, postgresql);
  return additionalSpaces +
 `${generateDefaultValue(defaultValue, value, currentColumnType, columnName, optionalName)}
  ${columnName}: ${currentColumnType};
  `;
};

const generateRelationsList = (modelName, type, name, relationName, baseName, params, cascadeDeleteArr, polymorphicRelations) => {
  return selectCurrentRelation(modelName, type, name, relationName, baseName, params, cascadeDeleteArr, polymorphicRelations);
};

const generateArrayRelations = (relations, arrayNamesForUpdate) => {
  const arrayModels = [];
  const arrayThroughModels = [];

  Object.keys(relations).map((relation, index) => {
    const { type, through, model: modelName, polymorphic, keyThrough, foreignKey } = Object.values(relations)[index];
    const relationName = arrayNamesForUpdate.includes(relation) ? capitalizeFirstLetter(relation) : relation;
    if (through) {
      arrayThroughModels.push({
        modelName: through,
        relation: relationName,
        params: {
          polymorphic,
          through,
          keyThrough,
          foreignKey,
        },
        type: 'manyToMany',
      });
    } else {
      arrayModels.push({
        modelName,
        relation: relationName,
        params: {
          polymorphic,
          through,
          keyThrough,
          foreignKey,
        },
        type,
      });
    }
  });
  return [...arrayModels, ...arrayThroughModels];
};

module.exports = {
  IMPORT_LINE,
  IMPORT_POLYMORPHIC_LINE,
  IMPORT_POLYMORPHIC_INTERFACE_LINE,
  entityLine,
  CLOSE_ENTITY,
  generateClassNameEntity,
  generateImportModel,
  generateColumnLines,
  generateRelationsList,
  generateArrayRelations,
  generatePrimaryColumn,
};